@@ -149,10 +149,13 @@ class TestUndercloud(BaseTestCase):
     @mock.patch('instack_undercloud.undercloud._check_sysctl')
     @mock.patch('instack_undercloud.undercloud._validate_network')
     @mock.patch('instack_undercloud.undercloud._validate_no_ip_change')
-    def test_validate_configuration(self, mock_vnic, mock_validate_network,
+    @mock.patch('instack_undercloud.undercloud._validate_passwords_file')
+    def test_validate_configuration(self, mock_vpf, mock_vnic,
+                                    mock_validate_network,
                                     mock_check_memory, mock_check_hostname,
                                     mock_check_sysctl):
         undercloud._validate_configuration()
+        self.assertTrue(mock_vpf.called)
         self.assertTrue(mock_vnic.called)
         self.assertTrue(mock_validate_network.called)
         self.assertTrue(mock_check_memory.called)
@@ -325,6 +328,22 @@ class TestNoIPChange(BaseTestCase):
         undercloud._validate_no_ip_change()
 
 
+@mock.patch('os.path.isfile')
+class TestPasswordsFileExists(BaseTestCase):
+    def test_new_install(self, mock_isfile):
+        mock_isfile.side_effect = [False]
+        undercloud._validate_passwords_file()
+
+    def test_update_exists(self, mock_isfile):
+        mock_isfile.side_effect = [True, True]
+        undercloud._validate_passwords_file()
+
+    def test_update_missing(self, mock_isfile):
+        mock_isfile.side_effect = [True, False]
+        self.assertRaises(validator.FailedValidation,
+                          undercloud._validate_passwords_file)
+
+
 class TestGenerateEnvironment(BaseTestCase):
     def setUp(self):
         super(TestGenerateEnvironment, self).setUp()
@@ -398,7 +417,7 @@ class TestGenerateEnvironment(BaseTestCase):
         env = undercloud._generate_environment('.')
         # Just spot check, we don't want to replicate the entire opt list here
         self.assertEqual(env['INSPECTION_COLLECTORS'],
-                         'default,extra-hardware,logs')
+                         'default,extra-hardware,numa-topology,logs')
         self.assertEqual('192.168.24.1/24', env['PUBLIC_INTERFACE_IP'])
         self.assertEqual('192.168.24.1', env['LOCAL_IP'])
         # The list is generated from a set, so we can't rely on ordering.
@@ -452,7 +471,7 @@ class TestGenerateEnvironment(BaseTestCase):
         env = undercloud._generate_environment('.')
         endpoint_vars = {k: v for (k, v) in env.items()
                          if k.startswith('UNDERCLOUD_ENDPOINT')}
-        self.assertEqual(90, len(endpoint_vars))
+        self.assertEqual(93, len(endpoint_vars))
         # Spot check one service
         self.assertEqual('http://192.168.24.1:5000',
                          env['UNDERCLOUD_ENDPOINT_KEYSTONE_PUBLIC'])
@@ -914,6 +933,43 @@ class TestPostConfig(base.BaseTestCase):
         undercloud._delete_default_flavors(mock_instance)
         mock_instance.flavors.delete.assert_called_once_with('8ar')
 
+    @mock.patch('os.path.isfile', return_value=True)
+    @mock.patch('os.listdir')
+    @mock.patch('instack_undercloud.undercloud._create_mistral_config_'
+                'environment')
+    @mock.patch('instack_undercloud.undercloud._migrate_plans')
+    @mock.patch('instack_undercloud.undercloud._create_default_plan')
+    def test_post_config_mistral(self, mock_create, mock_migrate, mock_cmce,
+                                 mock_listdir, mock_isfile):
+        instack_env = {}
+        mock_mistral = mock.Mock()
+        mock_swift = mock.Mock()
+        mock_swift.get_account.return_value = [None, [{'name': 'hut8'}]]
+
+        mock_workbooks = [mock.Mock() for m in range(2)]
+        mock_workbooks[0].name = 'foo'
+        mock_workbooks[1].name = 'tripleo.bar'
+        mock_mistral.workbooks.list.return_value = mock_workbooks
+        mock_workflows = [mock.Mock() for m in range(2)]
+        mock_workflows[0].name = 'foo'
+        mock_workflows[1].name = 'tripleo.bar'
+        mock_mistral.workflows.list.return_value = mock_workflows
+        mock_listdir.return_value = ['foo.yaml', 'bar.yaml']
+        undercloud._post_config_mistral(instack_env, mock_mistral, mock_swift)
+        self.assertEqual([mock.call('tripleo.bar')],
+                         mock_mistral.workbooks.delete.mock_calls)
+        self.assertEqual([mock.call('tripleo.bar')],
+                         mock_mistral.workflows.delete.mock_calls)
+        self.assertEqual([mock.call(undercloud.PATHS.WORKBOOK_PATH +
+                                    '/foo.yaml'),
+                         mock.call(undercloud.PATHS.WORKBOOK_PATH +
+                                   '/bar.yaml')],
+                         mock_mistral.workbooks.create.mock_calls)
+        mock_cmce.assert_called_once_with(instack_env, mock_mistral)
+        mock_migrate.assert_called_once_with(mock_mistral, mock_swift,
+                                             ['hut8'])
+        mock_create.assert_called_once_with(mock_mistral, ['hut8'])
+
 
 class TestUpgradeFact(base.BaseTestCase):
     @mock.patch('instack_undercloud.undercloud._run_command')
