@@ -70,6 +70,10 @@ class Paths(object):
     def LOG_FILE(self):
         return os.path.expanduser('~/.instack/install-undercloud.log')
 
+    @property
+    def WORKBOOK_PATH(self):
+        return '/usr/share/openstack-tripleo-common/workbooks'
+
 
 PATHS = Paths()
 DEFAULT_LOG_LEVEL = logging.DEBUG
@@ -151,6 +155,14 @@ _opts = [
     cfg.ListOpt('undercloud_ntp_servers',
                 default=[],
                 help=('List of ntp servers to use.')),
+    cfg.StrOpt('overcloud_domain_name',
+               default='',
+               help=('DNS domain name to use when deploying the overcloud. '
+                     'The overcloud parameter "CloudDomain" must be set to a '
+                     'matching value. When unset, the installation will try '
+                     'to use the undercloud\'s domain name, and if that fails '
+                     'it\'ll use the default from neutron.conf.')
+               ),
     cfg.StrOpt('undercloud_service_certificate',
                default='',
                help=('Certificate file to use for OpenStack service SSL '
@@ -646,14 +658,23 @@ def _check_memory():
         raise RuntimeError('Insufficient memory available')
 
 
+def _check_ipv6_enabled():
+    """Test if IPv6 is enabled
+
+    If /proc/net/if_inet6 exist ipv6 sysctl settings are available.
+    """
+    return os.path.isfile('/proc/net/if_inet6')
+
+
 def _check_sysctl():
     """Check sysctl option availability
 
     The undercloud will not install properly if some of the expected sysctl
     values are not available to be set.
     """
-    options = ['net.ipv4.ip_forward', 'net.ipv4.ip_nonlocal_bind',
-               'net.ipv6.ip_nonlocal_bind']
+    options = ['net.ipv4.ip_forward', 'net.ipv4.ip_nonlocal_bind']
+    if _check_ipv6_enabled():
+        options.append('net.ipv6.ip_nonlocal_bind')
 
     not_available = []
     for option in options:
@@ -706,6 +727,22 @@ def _validate_no_ip_change():
         raise validator.FailedValidation(message)
 
 
+def _validate_passwords_file():
+    """Disallow updates if the passwords file is missing
+
+    If the undercloud was already deployed, the passwords file needs to be
+    present so passwords that can't be changed are persisted.  If the file
+    is missing it will break the undercloud, so we should fail-fast and let
+    the user know about the problem.
+    """
+    if (os.path.isfile(os.path.expanduser('~/stackrc')) and
+            not os.path.isfile(PATHS.PASSWORD_PATH)):
+        message = ('The %s file is missing.  This will cause all service '
+                   'passwords to change and break the existing undercloud. ' %
+                   PATHS.PASSWORD_PATH)
+        raise validator.FailedValidation(message)
+
+
 def _validate_configuration():
     try:
         _check_hostname()
@@ -713,6 +750,7 @@ def _validate_configuration():
         _check_sysctl()
         _validate_network()
         _validate_no_ip_change()
+        _validate_passwords_file()
     except RuntimeError as e:
         LOG.error('An error occurred during configuration validation, '
                   'please check your host configuration and try again. '
@@ -839,6 +877,10 @@ def _generate_endpoints(instack_env):
             '%s://%s:%d',
             {'host': public_host, 'port': 5050, 'ssl_port': 13050},
             {'host': internal_host, 'port': 5050}),
+        ('ironic_inspector-ui-config',
+            '%s://%s:%d/ironic-inspector',
+            {'host': public_host, 'port': 3000, 'ssl_port': 443},
+            {'host': internal_host, 'port': 3000}),
         ('aodh',
             '%s://%s:%d',
             {'host': public_host, 'port': 8042, 'ssl_port': 13042},
@@ -849,8 +891,8 @@ def _generate_endpoints(instack_env):
             {'host': internal_host, 'port': 8041}),
         ('panko',
             '%s://%s:%d',
-            {'host': public_host, 'port': 8779, 'ssl_port': 13779},
-            {'host': internal_host, 'port': 8779}),
+            {'host': public_host, 'port': 8977, 'ssl_port': 13779},
+            {'host': internal_host, 'port': 8977}),
         ('mistral',
             '%s://%s:%d/v2',
             {'host': public_host, 'port': 8989, 'ssl_port': 13989},
@@ -1002,7 +1044,7 @@ class InstackEnvironment(dict):
                     'TRIPLEO_INSTALL_USER', 'TRIPLEO_UNDERCLOUD_CONF_FILE',
                     'TRIPLEO_UNDERCLOUD_PASSWORD_FILE',
                     'ENABLED_POWER_INTERFACES',
-                    'ENABLED_MANAGEMENT_INTERFACES'}
+                    'ENABLED_MANAGEMENT_INTERFACES', 'SYSCTL_SETTINGS'}
     """The variables we calculate in _generate_environment call."""
 
     PUPPET_KEYS = DYNAMIC_KEYS | {opt.name.upper() for _, group in list_opts()
@@ -1027,6 +1069,14 @@ def _make_list(values):
     return '[%s]' % ', '.join('"%s"' % item for item in values)
 
 
+def _generate_sysctl_settings():
+    sysctl_settings = {}
+    sysctl_settings.update({"net.ipv4.ip_nonlocal_bind": {"value": 1}})
+    if _check_ipv6_enabled():
+        sysctl_settings.update({"net.ipv6.ip_nonlocal_bind": {"value": 1}})
+    return json.dumps(sysctl_settings)
+
+
 def _generate_environment(instack_root):
     """Generate an environment dict for instack
 
@@ -1092,7 +1142,8 @@ def _generate_environment(instack_root):
         raise RuntimeError('inspection_extras must be enabled for '
                            'inspection_runbench to work')
     if CONF.inspection_extras:
-        instack_env['INSPECTION_COLLECTORS'] = 'default,extra-hardware,logs'
+        instack_env['INSPECTION_COLLECTORS'] = ('default,extra-hardware,'
+                                                'numa-topology,logs')
     else:
         instack_env['INSPECTION_COLLECTORS'] = 'default,logs'
 
@@ -1133,6 +1184,8 @@ def _generate_environment(instack_root):
     instack_env['ENABLED_POWER_INTERFACES'] = enabled_interfaces
     instack_env['ENABLED_MANAGEMENT_INTERFACES'] = enabled_interfaces
 
+    instack_env['SYSCTL_SETTINGS'] = _generate_sysctl_settings()
+
     if CONF.docker_registry_mirror:
         instack_env['DOCKER_REGISTRY_MIRROR'] = CONF.docker_registry_mirror
 
@@ -1385,10 +1438,7 @@ def _migrate_plans(mistral, swift, plans):
             env = mistral.environments.get(plan).variables
             yaml_string = yaml.safe_dump(env, default_flow_style=False)
             swift.put_object(plan, plan_env_filename, yaml_string)
-            # TODO(jpichon): delete env from Mistral once triple-common
-            # change merges (https://review.openstack.org/#/c/452291/).
-            # Right now plan operations still expect a Mistral environment.
-            # mistral.environments.delete(plan)
+            mistral.environments.delete(plan)
 
 
 def _create_default_plan(mistral, plans, timeout=360):
@@ -1434,6 +1484,18 @@ def _prepare_ssh_environment(mistral):
 
 
 def _post_config_mistral(instack_env, mistral, swift):
+    LOG.info('Configuring Mistral workbooks')
+    for workbook in [w for w in mistral.workbooks.list()
+                     if 'tripleo' in w.name]:
+        mistral.workbooks.delete(workbook.name)
+    for workflow in [w for w in mistral.workflows.list()
+                     if 'tripleo' in w.name]:
+        mistral.workflows.delete(workflow.name)
+    for workbook in [f for f in os.listdir(PATHS.WORKBOOK_PATH)
+                     if os.path.isfile(os.path.join(PATHS.WORKBOOK_PATH, f))]:
+        mistral.workbooks.create(os.path.join(PATHS.WORKBOOK_PATH, workbook))
+    LOG.info('Mistral workbooks configured successfully')
+
     plans = [container["name"] for container in swift.get_account()[1]]
 
     _create_mistral_config_environment(instack_env, mistral)
@@ -1581,3 +1643,17 @@ def install(instack_root, upgrade=False):
                  {'undercloud_operation': undercloud_operation,
                   'password_path': PATHS.PASSWORD_PATH,
                   'stackrc_path': os.path.expanduser('~/stackrc')})
+
+
+def pre_upgrade():
+    _configure_logging(DEFAULT_LOG_LEVEL, PATHS.LOG_FILE)
+    args = ['sudo', 'systemctl', 'stop', 'openstack-*', 'neutron-*',
+            'openvswitch', 'httpd']
+    LOG.info('Stopping OpenStack and related services')
+    _run_live_command(args, name='systemctl stop')
+    LOG.info('Services stopped successfully')
+
+    args = ['sudo', 'yum', 'update', '-y']
+    LOG.info('Updating full system')
+    _run_live_command(args, name='yum update')
+    LOG.info('Update completed successfully')
